@@ -1,4 +1,4 @@
-/* 
+/*
 
    this is a pretty direct translation (with only vague understanding,
    unfortunately) of aubrey jaffer's most recent O(NP) edit-script
@@ -16,7 +16,7 @@
    computational biology. apparantly it's submitted for publication there
    too.
 
-   --- 
+   ---
 
    "differ.scm" O(NP) Sequence Comparison Algorithm.
    Copyright (C) 2001, 2002, 2003 Aubrey Jaffer
@@ -38,19 +38,28 @@
 
 */
 
-#include "lcs.hh"
-
-#include <vector>
 #include <algorithm>
 #include <iostream>
+#include <vector>
+
+#include "lcs.hh"
 #include "sanity.hh"
 
+using std::back_insert_iterator;
+using std::back_inserter;
+using std::copy;
+using std::iterator_traits;
+using std::max;
+using std::min;
+using std::sort;
+using std::vector;
+
 struct work_vec
 {
   long lo;
   long hi;
-  static std::vector<long, QA(long)> vec;
-  work_vec(long lo, long hi) : 
+  static vector<long, QA(long)> vec;
+  work_vec(long lo, long hi) :
     lo(lo), hi(hi)
   {
     // I(hi >= lo);
@@ -59,34 +68,34 @@ struct work_vec
     vec.assign(len, -1);
   }
 
-  inline long & operator[](long t) 
+  inline long & operator[](long t)
   {
     // I(t >= lo && t <= hi);
-    return vec[t-lo]; 
+    return vec[t-lo];
   }
 };
 
-std::vector<long, QA(long)> work_vec::vec;
+vector<long, QA(long)> work_vec::vec;
 
 template <typename A,
           typename B,
-          typename LCS>        
+          typename LCS>
 struct jaffer_edit_calculator
 {
 
-  typedef std::vector<long, QA(long)> cost_vec;
-  typedef std::vector<long, QA(long)> edit_vec;
+  typedef vector<long, QA(long)> cost_vec;
+  typedef vector<long, QA(long)> edit_vec;
 
   template <typename T>
   struct subarray
   {
-    typedef typename std::iterator_traits<T>::value_type vt;
+    typedef typename iterator_traits<T>::value_type vt;
 
     T base;     // underlying representation
     long start; // current extent
     long end;   // current extent
-    
-    subarray(T b, long s, long e) : 
+
+    subarray(T b, long s, long e) :
       base(b), start(s), end(e) {}
 
     inline long size() const
@@ -96,12 +105,12 @@ struct jaffer_edit_calculator
       else
         return end - start;
     }
-    
+
     inline subarray subset(long s, long e) const
     {
-      return subarray(base + std::min(start, end), s, e);
+      return subarray(base + min(start, end), s, e);
     }
-    
+
     inline vt const & operator[](size_t idx) const
     {
       if (end < start)
@@ -109,30 +118,30 @@ struct jaffer_edit_calculator
       else
           return *(base + (start + idx));
     }
-  };    
+  };
 
-  static long run(work_vec & fp, long k, 
+  static long run(work_vec & fp, long k,
                   subarray<A> const & a, long m,
                   subarray<B> const & b, long n,
                   cost_vec & CC, long p)
   {
     long cost = k + 2*p;
-    
+
     // do the run
-    long y = std::max(fp[k-1]+1, fp[k+1]);
+    long y = max(fp[k-1]+1, fp[k+1]);
     long x = y - k;
 
     I(y >= 0);
     I(x >= 0);
 
     while (true)
-      {        
+      {
         // record costs along the way
         long xcst = m - x;
         if (y < static_cast<long>(CC.size()) && xcst >= 0)
           {
-            CC[y] = std::min(xcst + cost, CC[y]); 
-          }        
+            CC[y] = min(xcst + cost, CC[y]);
+          }
         if (x < m && y < n && a[x] == b[y])
           {
             ++x; ++y;
@@ -146,7 +155,7 @@ struct jaffer_edit_calculator
   }
 
   // 'compare' here is the core myers, manber and miller algorithm.
-  static long compare(cost_vec & costs, 
+  static long compare(cost_vec & costs,
                       subarray<A> const & a, long m,
                       subarray<B> const & b, long n,
                       long p_lim,
@@ -184,11 +193,11 @@ struct jaffer_edit_calculator
 
     return delta + 2*p;
   }
-  
+
   static long divide_and_conquer(subarray<A> const & a, long start_a, long end_a,
                                  subarray<B> const & b, long start_b, long end_b,
                                  edit_vec & edits,
-                                 unsigned long edx, 
+                                 unsigned long edx,
                                  long polarity,
                                  long p_lim)
   {
@@ -205,17 +214,17 @@ struct jaffer_edit_calculator
     I(end_a <= a.size());
     I(end_b >= 0);
     I(end_b <= b.size());
-    
+
     cost_vec cc(len_b + 1, len_a + len_b);
     cost_vec rr(len_b + 1, len_a + len_b);
 
-    compare (cc, 
+    compare (cc,
              a.subset(start_a, mid_a), (mid_a - start_a),
-             b.subset(start_b, end_b), len_b, std::min(p_lim, len_a));
+             b.subset(start_b, end_b), len_b, min(p_lim, len_a));
 
-    compare (rr, 
+    compare (rr,
              a.subset(end_a, mid_a), (end_a - mid_a),
-             b.subset(end_b, start_b), len_b, std::min(p_lim, len_a));
+             b.subset(end_b, start_b), len_b, min(p_lim, len_a));
 
     long b_split = mid_split(len_a, len_b, rr, cc, tcst);
 
@@ -224,22 +233,22 @@ struct jaffer_edit_calculator
 
     long cost_c = diff_to_et (a, start_a, mid_a,
                               b, start_b, start_b + b_split,
-                              edits, edx, polarity, 
+                              edits, edx, polarity,
                               (est_c - (b_split - (mid_a - start_a))) / 2);
 
     I(cost_c == est_c);
-    
+
     long cost_r = diff_to_et (a, mid_a, end_a,
                               b, start_b + b_split, end_b,
                               edits, est_c + edx, polarity,
                               (est_r - ((len_b - b_split) - (end_a - mid_a))) / 2);
 
     I(cost_r == est_r);
-    
+
     return est_r + est_c;
   }
-  
-  static long mid_split(long m, long n, 
+
+  static long mid_split(long m, long n,
                         cost_vec const & rr,
                         cost_vec const & cc,
                         long cost)
@@ -250,16 +259,16 @@ struct jaffer_edit_calculator
       {
         I (rdx >= 0);
 
-        if (cost == (cc[rdx] + rr[n-rdx])) 
+        if (cost == (cc[rdx] + rr[n-rdx]))
             return rdx;
         if (cost == (cc[cdx] + rr[n-cdx]))
-            return cdx;          
+            return cdx;
         --rdx;
         ++cdx;
       }
   }
-  
-  
+
+
   static void order_edits(edit_vec const & edits,
                           long sign,
                           edit_vec & nedits)
@@ -267,39 +276,39 @@ struct jaffer_edit_calculator
     nedits.clear();
     nedits.resize(edits.size());
     long cost = edits.size();
-    
+
     if (cost == 0)
       {
         nedits = edits;
         return;
       }
-    
+
     edit_vec sedits = edits;
-    std::sort(sedits.begin(), sedits.end());
+    sort(sedits.begin(), sedits.end());
 
     long idx0;
     for (idx0 = 0; idx0 < cost && sedits[idx0] < 0; ++idx0);
-    long len_a = std::max(0L, -sedits[0]);
+    long len_a = max(0L, -sedits[0]);
     long len_b = sedits[cost-1];
-    
+
     long ddx = idx0 - 1;
     long idx = idx0;
     long ndx = 0;
     long adx = 0;
     long bdx = 0;
-    
+
     while (bdx < len_b || adx < len_a)
       {
 
         long del = (ddx < 0) ? 0 : sedits[ddx];
         long ins = (idx >= cost) ? 0 : sedits[idx];
-        
+
         if (del < 0 && adx >= (-1 - del) &&
             ins > 0 && bdx >= (-1 + ins))
           {
             nedits[ndx] = del;
             nedits[ndx+1] = ins;
-            --ddx; ++idx; ndx += 2; ++adx; ++bdx;              
+            --ddx; ++idx; ndx += 2; ++adx; ++bdx;
           }
         else if (del < 0 && adx >= (-1 - del))
           {
@@ -317,11 +326,11 @@ struct jaffer_edit_calculator
           }
       }
   }
-  
+
   // trims and calls diff_to_ez
   static long diff_to_et(subarray<A> const & a, long start_a, long end_a,
                          subarray<B> const & b, long start_b, long end_b,
-                         std::vector<long, QA(long)> & edits,
+                         vector<long, QA(long)> & edits,
                          long edx,
                          long polarity,
                          long p_lim)
@@ -339,31 +348,31 @@ struct jaffer_edit_calculator
     I(end_a - start_a >= p_lim);
 
     long bsx, bdx, asx, adx;
-    
+
     for (bdx = end_b - 1, adx = end_a - 1;
-         (start_b <= bdx) && (start_a <= adx) && (a[adx] == b[bdx]);           
+         (start_b <= bdx) && (start_a <= adx) && (a[adx] == b[bdx]);
          --bdx, --adx);
-    
+
     for (bsx = start_b, asx = start_a;
          (bsx < bdx) && (asx < adx) && (a[asx] == b[bsx]);
          ++bsx, ++asx);
-    
+
     // we've trimmed; now call diff_to_ez.
-    
+
     long delta = (bdx - bsx) - (adx - asx);
     if (delta < 0)
-      return diff_to_ez (b, bsx, bdx+1, 
+      return diff_to_ez (b, bsx, bdx+1,
                          a, asx, adx+1,
                          edits, edx, -polarity, delta + p_lim);
     else
       return diff_to_ez (a, asx, adx+1,
-                         b, bsx, bdx+1, 
-                         edits, edx, polarity, p_lim);              
+                         b, bsx, bdx+1,
+                         edits, edx, polarity, p_lim);
   }
-  
+
   static long diff_to_ez(subarray<A> const & a, long start_a, long end_a,
                          subarray<B> const & b, long start_b, long end_b,
-                         std::vector<long, QA(long)> & edits,
+                         vector<long, QA(long)> & edits,
                          long edx1,
                          long polarity,
                          long p_lim)
@@ -389,26 +398,26 @@ struct jaffer_edit_calculator
         // A == B, no edits
         if (len_a == len_b)
           return 0;
-        
+
         long adx = start_a;
         long bdx = start_b;
         long edx0 = edx1;
-        
+
         while (true)
-          {               
+          {
             if (bdx >= end_b)
               return len_b - len_a;
-            
+
             if (adx >= end_a)
               {
-                for (long idx = bdx, edx = edx0; 
-                     idx < end_b; 
+                for (long idx = bdx, edx = edx0;
+                     idx < end_b;
                      ++idx, ++edx)
                   edits[edx] = polarity * (idx+1);
-                
-                return len_b - len_a;                  
+
+                return len_b - len_a;
               }
-            
+
             if (a[adx] == b[bdx])
               {
                 ++adx; ++bdx;
@@ -420,22 +429,22 @@ struct jaffer_edit_calculator
               }
           }
       }
-    
+
     // easy case #2: delete all A, insert all B
-    else if (len_a <= p_lim) 
+    else if (len_a <= p_lim)
       {
         I(len_a == p_lim);
 
         long edx0 = edx1;
         for (long idx = start_a; idx < end_a; ++idx, ++edx0)
           edits[edx0] = polarity * (-1 - idx);
-        
+
         for (long jdx = start_b; jdx < end_b; ++jdx, ++edx0)
           edits[edx0] = polarity * (jdx + 1);
-        
+
         return len_a + len_b;
       }
-    
+
     // hard case: recurse on subproblems
     else
       {
@@ -444,10 +453,10 @@ struct jaffer_edit_calculator
                                    edits, edx1, polarity, p_lim);
       }
   }
-  
+
   static void diff_to_edits(subarray<A> const & a, long m,
                             subarray<B> const & b, long n,
-                            std::vector<long, QA(long)> & edits,
+                            vector<long, QA(long)> & edits,
                             long p_lim)
   {
     I(m <= n);
@@ -456,23 +465,23 @@ struct jaffer_edit_calculator
 
     edits.clear();
     edits.resize(edit_distance, 0);
-    long cost = diff_to_et(a, 0, m, 
-                           b, 0, n, 
+    long cost = diff_to_et(a, 0, m,
+                           b, 0, n,
                            edits, 0, 1, (edit_distance - (n-m)) / 2);
     I(cost == edit_distance);
   }
-  
-  static void edits_to_lcs (std::vector<long, QA(long)> const & edits, 
+
+  static void edits_to_lcs (vector<long, QA(long)> const & edits,
                             subarray<A> const a, long m, long n,
                             LCS output)
   {
     long edx = 0, sdx = 0, adx = 0;
-    typedef typename std::iterator_traits<A>::value_type vt;
-    std::vector<vt, QA(vt)> lcs(((m + n) - edits.size()) / 2);
+    typedef typename iterator_traits<A>::value_type vt;
+    vector<vt, QA(vt)> lcs(((m + n) - edits.size()) / 2);
     while (true)
       {
         long edit = (edx < static_cast<long>(edits.size())) ? edits[edx] : 0;
-        
+
         if (adx >= m)
           break;
         else if (edit > 0)
@@ -485,7 +494,7 @@ struct jaffer_edit_calculator
           { lcs[sdx++] = a[adx++]; }
       }
 
-    std::copy(lcs.begin(), lcs.end(), output);
+    copy(lcs.begin(), lcs.end(), output);
   }
 };
 
@@ -496,7 +505,7 @@ template <typename A,
 void _edit_script(A begin_a, A end_a,
                   B begin_b, B end_b,
                   long p_lim,
-                  std::vector<long, QA(long)> & edits_out,
+                  vector<long, QA(long)> & edits_out,
                   LCS ignored_out)
 {
   typedef jaffer_edit_calculator<A,B,LCS> calc_t;
@@ -557,30 +566,38 @@ void _longest_common_subsequence(A begin_a, A end_a,
 }
 
 
-void 
-longest_common_subsequence(std::vector<long, QA(long)>::const_iterator begin_a,
-                           std::vector<long, QA(long)>::const_iterator end_a,
-                           std::vector<long, QA(long)>::const_iterator begin_b,
-                           std::vector<long, QA(long)>::const_iterator end_b,
+void
+longest_common_subsequence(vector<long, QA(long)>::const_iterator begin_a,
+                           vector<long, QA(long)>::const_iterator end_a,
+                           vector<long, QA(long)>::const_iterator begin_b,
+                           vector<long, QA(long)>::const_iterator end_b,
                            long p_lim,
-                           std::back_insert_iterator< std::vector<long, QA(long)> > lcs)
+                           back_insert_iterator< vector<long, QA(long)> > lcs)
 {
   _longest_common_subsequence(begin_a, end_a,
                               begin_b, end_b,
                               p_lim, lcs);
 }
 
-void 
-edit_script(std::vector<long, QA(long)>::const_iterator begin_a,
-            std::vector<long, QA(long)>::const_iterator end_a,
-            std::vector<long, QA(long)>::const_iterator begin_b,
-            std::vector<long, QA(long)>::const_iterator end_b,
+void
+edit_script(vector<long, QA(long)>::const_iterator begin_a,
+            vector<long, QA(long)>::const_iterator end_a,
+            vector<long, QA(long)>::const_iterator begin_b,
+            vector<long, QA(long)>::const_iterator end_b,
             long p_lim,
-            std::vector<long, QA(long)> & edits_out)
+            vector<long, QA(long)> & edits_out)
 {
-  std::vector<long, QA(long)> lcs;
-  _edit_script(begin_a, end_a, 
-               begin_b, end_b, 
+  vector<long, QA(long)> lcs;
+  _edit_script(begin_a, end_a,
+               begin_b, end_b,
                p_lim, edits_out,
                back_inserter(lcs));
 }
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
