@@ -1,14 +1,19 @@
-// -*- mode: C++; c-file-style: "gnu"; indent-tabs-mode: nil -*-
-// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
+// Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
-#include <config.h>
+#include "config.h"
 
 #include "popt/popt.h"
 #include <cstdio>
+#ifndef _MSC_VER
 #include <strings.h>
+#endif
 #include <iterator>
 #include <iostream>
 #include <fstream>
@@ -16,26 +21,34 @@
 #include <locale.h>
 
 #include <stdlib.h>
-#ifdef WIN32
-#include <libintl.h>
-#endif
+
+#include <boost/filesystem/convenience.hpp>
+#include <boost/filesystem/path.hpp>
 
 #include "botan/botan.h"
 
+#include "i18n.h"
+
 #include "app_state.hh"
 #include "commands.hh"
 #include "sanity.hh"
 #include "cleanup.hh"
 #include "file_io.hh"
-#include "transforms.hh"
+#include "charset.hh"
 #include "ui.hh"
 #include "mt_version.hh"
 #include "options.hh"
 #include "paths.hh"
 
-// main option processing and exception handling code
+using std::cout;
+using std::endl;
+using std::ios_base;
+using std::ostringstream;
+using std::set;
+using std::string;
+using std::vector;
 
-using namespace std;
+// main option processing and exception handling code
 
 char * argstr = NULL;
 long arglong = 0;
@@ -56,23 +69,28 @@ struct poptOption coptions[] =
     {"date", 0, POPT_ARG_STRING, &argstr, OPT_DATE, gettext_noop("override date/time for commit"), NULL},
     {"author", 0, POPT_ARG_STRING, &argstr, OPT_AUTHOR, gettext_noop("override author for commit"), NULL},
     {"depth", 0, POPT_ARG_LONG, &arglong, OPT_DEPTH, gettext_noop("limit the number of levels of directories to descend"), NULL},
-    {"last", 0, POPT_ARG_LONG, &arglong, OPT_LAST, gettext_noop("limit the log output to the given number of entries"), NULL},
+    {"last", 0, POPT_ARG_LONG, &arglong, OPT_LAST, gettext_noop("limit log output to the last number of entries"), NULL},
+    {"next", 0, POPT_ARG_LONG, &arglong, OPT_NEXT, gettext_noop("limit log output to the next number of entries"), NULL},
     {"pid-file", 0, POPT_ARG_STRING, &argstr, OPT_PIDFILE, gettext_noop("record process id of server"), NULL},
     {"brief", 0, POPT_ARG_NONE, NULL, OPT_BRIEF, gettext_noop("print a brief version of the normal output"), NULL},
     {"diffs", 0, POPT_ARG_NONE, NULL, OPT_DIFFS, gettext_noop("print diffs along with logs"), NULL},
-    {"no-merges", 0, POPT_ARG_NONE, NULL, OPT_NO_MERGES, gettext_noop("skip merges when printing logs"), NULL},
+    {"no-merges", 0, POPT_ARG_NONE, NULL, OPT_NO_MERGES, gettext_noop("exclude merges when printing logs"), NULL},
     {"set-default", 0, POPT_ARG_NONE, NULL, OPT_SET_DEFAULT, gettext_noop("use the current arguments as the future default"), NULL},
     {"exclude", 0, POPT_ARG_STRING, &argstr, OPT_EXCLUDE, gettext_noop("leave out anything described by its argument"), NULL},
     {"unified", 0, POPT_ARG_NONE, NULL, OPT_UNIFIED_DIFF, gettext_noop("use unified diff format"), NULL},
     {"context", 0, POPT_ARG_NONE, NULL, OPT_CONTEXT_DIFF, gettext_noop("use context diff format"), NULL},
     {"external", 0, POPT_ARG_NONE, NULL, OPT_EXTERNAL_DIFF, gettext_noop("use external diff hook for generating diffs"), NULL},
     {"diff-args", 0, POPT_ARG_STRING, &argstr, OPT_EXTERNAL_DIFF_ARGS, gettext_noop("argument to pass external diff hook"), NULL},
-    {"lca", 0, POPT_ARG_NONE, NULL, OPT_LCA, gettext_noop("use least common ancestor as ancestor for merge"), NULL},
     {"execute", 'e', POPT_ARG_NONE, NULL, OPT_EXECUTE, gettext_noop("perform the associated file operation"), NULL},
-    {"bind", 0, POPT_ARG_STRING, &argstr, OPT_BIND, gettext_noop("address:port to listen on (default :5253)"), NULL},
-    {"missing", 0, POPT_ARG_NONE, NULL, OPT_MISSING, gettext_noop("perform the operations for files missing from working directory"), NULL},
-    {"unknown", 0, POPT_ARG_NONE, NULL, OPT_UNKNOWN, gettext_noop("perform the operations for unknown files from working directory"), NULL},
+    {"bind", 0, POPT_ARG_STRING, &argstr, OPT_BIND, gettext_noop("address:port to listen on (default :4691)"), NULL},
+    {"missing", 0, POPT_ARG_NONE, NULL, OPT_MISSING, gettext_noop("perform the operations for files missing from workspace"), NULL},
+    {"unknown", 0, POPT_ARG_NONE, NULL, OPT_UNKNOWN, gettext_noop("perform the operations for unknown files from workspace"), NULL},
     {"key-to-push", 0, POPT_ARG_STRING, &argstr, OPT_KEY_TO_PUSH, gettext_noop("push the specified key even if it hasn't signed anything"), NULL},
+    {"stdio", 0, POPT_ARG_NONE, NULL, OPT_STDIO, gettext_noop("serve netsync on stdio"), NULL},
+    {"no-transport-auth", 0, POPT_ARG_NONE, NULL, OPT_NO_TRANSPORT_AUTH, gettext_noop("disable transport authentication"), NULL},
+    {"drop-attr", 0, POPT_ARG_STRING, &argstr, OPT_DROP_ATTR, gettext_noop("when rosterifying, drop attrs entries with the given key"), NULL},
+    {"no-files", 0, POPT_ARG_NONE, NULL, OPT_NO_FILES, gettext_noop("exclude files when printing logs"), NULL},
+    {"recursive", 'R', POPT_ARG_NONE, NULL, OPT_RECURSIVE, gettext_noop("also operate on the contents of any listed directories"), NULL},
     { NULL, 0, 0, NULL, 0, NULL, NULL }
   };
 
@@ -83,18 +101,20 @@ struct poptOption options[] =
 
     {"debug", 0, POPT_ARG_NONE, NULL, OPT_DEBUG, gettext_noop("print debug log to stderr while running"), NULL},
     {"dump", 0, POPT_ARG_STRING, &argstr, OPT_DUMP, gettext_noop("file to dump debugging log to, on failure"), NULL},
-    {"quiet", 0, POPT_ARG_NONE, NULL, OPT_QUIET, gettext_noop("suppress log and progress messages"), NULL},
+    {"log", 0, POPT_ARG_STRING, &argstr, OPT_LOG, gettext_noop("file to write the log to"), NULL},
+    {"quiet", 0, POPT_ARG_NONE, NULL, OPT_QUIET, gettext_noop("suppress verbose, informational and progress messages"), NULL},
+    {"reallyquiet", 0, POPT_ARG_NONE, NULL, OPT_REALLYQUIET, gettext_noop("suppress warning, verbose, informational and progress messages"), NULL},
     {"help", 'h', POPT_ARG_NONE, NULL, OPT_HELP, gettext_noop("display help message"), NULL},
     {"version", 0, POPT_ARG_NONE, NULL, OPT_VERSION, gettext_noop("print version number, then exit"), NULL},
     {"full-version", 0, POPT_ARG_NONE, NULL, OPT_FULL_VERSION, gettext_noop("print detailed version number, then exit"), NULL},
     {"xargs", '@', POPT_ARG_STRING, &argstr, OPT_ARGFILE, gettext_noop("insert command line arguments taken from the given file"), NULL},
     {"ticker", 0, POPT_ARG_STRING, &argstr, OPT_TICKER, gettext_noop("set ticker style (count|dot|none)"), NULL},
     {"nostd", 0, POPT_ARG_NONE, NULL, OPT_NOSTD, gettext_noop("do not load standard lua hooks"), NULL},
-    {"norc", 0, POPT_ARG_NONE, NULL, OPT_NORC, gettext_noop("do not load ~/.monotone/monotonerc or MT/monotonerc lua files"), NULL},
+    {"norc", 0, POPT_ARG_NONE, NULL, OPT_NORC, gettext_noop("do not load ~/.monotone/monotonerc or _MTN/monotonerc lua files"), NULL},
     {"rcfile", 0, POPT_ARG_STRING, &argstr, OPT_RCFILE, gettext_noop("load extra rc file"), NULL},
     {"key", 'k', POPT_ARG_STRING, &argstr, OPT_KEY_NAME, gettext_noop("set key for signatures"), NULL},
     {"db", 'd', POPT_ARG_STRING, &argstr, OPT_DB_NAME, gettext_noop("set name of database"), NULL},
-    {"root", 0, POPT_ARG_STRING, &argstr, OPT_ROOT, gettext_noop("limit search for working copy to specified root"), NULL},
+    {"root", 0, POPT_ARG_STRING, &argstr, OPT_ROOT, gettext_noop("limit search for workspace to specified root"), NULL},
     {"verbose", 0, POPT_ARG_NONE, NULL, OPT_VERBOSE, gettext_noop("verbose completion output"), NULL},
     {"keydir", 0, POPT_ARG_STRING, &argstr, OPT_KEY_DIR, gettext_noop("set location of key store"), NULL},
     {"confdir", 0, POPT_ARG_STRING, &argstr, OPT_CONF_DIR, gettext_noop("set location of configuration directory"), NULL},
@@ -125,17 +145,17 @@ struct poptOption options[] =
 // in other words, this program should *never* unexpectedly terminate
 // without dumping some diagnostics.
 
-void 
-dumper() 
+void
+dumper()
 {
   if (!global_sanity.clean_shutdown)
     global_sanity.dump_buffer();
-  
+
   Botan::Init::deinitialize();
 }
 
 
-struct 
+struct
 utf8_argv
 {
   int argc;
@@ -158,7 +178,7 @@ utf8_argv
     }
   }
 
-  ~utf8_argv() 
+  ~utf8_argv()
   {
     if (argv != NULL)
       {
@@ -166,7 +186,7 @@ utf8_argv
           if (argv[i] != NULL)
             free(argv[i]);
         free(argv);
-      }    
+      }
   }
 };
 
@@ -232,7 +252,7 @@ coption_string(int o)
   return string();
 }
 
-int 
+int
 cpp_main(int argc, char ** argv)
 {
   int ret = 0;
@@ -251,32 +271,49 @@ cpp_main(int argc, char ** argv)
   try
   {
 
+  // set up some marked strings, so even if our logbuf overflows, we'll get
+  // this data in a crash.
+  string cmdline_string;
   {
-    std::ostringstream cmdline_ss;
+    ostringstream cmdline_ss;
     for (int i = 0; i < argc; ++i)
       {
         if (i)
           cmdline_ss << ", ";
         cmdline_ss << "'" << argv[i] << "'";
       }
-    L(F("command line: %s\n") % cmdline_ss.str());
+    cmdline_string = cmdline_ss.str();
   }
+  MM(cmdline_string);
+  L(FL("command line: %s") % cmdline_string);
 
-  L(F("set locale: LC_ALL=%s\n")
-    % (setlocale(LC_ALL, NULL) == NULL ? "n/a" : setlocale(LC_ALL, NULL)));
+  string locale_string = (setlocale(LC_ALL, NULL) == NULL ? "n/a" : setlocale(LC_ALL, NULL));
+  MM(locale_string);
+  L(FL("set locale: LC_ALL=%s") % locale_string);
+
+  string full_version_string;
+  get_full_version(full_version_string);
+  MM(full_version_string);
 
   // Set up secure memory allocation etc
   Botan::Init::initialize();
   Botan::set_default_allocator("malloc");
-  
+
   // decode all argv values into a UTF-8 array
 
   save_initial_path();
   utf8_argv uv(argc, argv);
 
+  // find base name of executable
+
+  string prog_path = fs::path(uv.argv[0]).leaf();
+  if (prog_path.rfind(".exe") == prog_path.size() - 4)
+    prog_path = prog_path.substr(0, prog_path.size() - 4);
+  utf8 prog_name(prog_path);
+
   // prepare for arg parsing
 
-  cleanup_ptr<poptContext, void> 
+  cleanup_ptr<poptContext, void>
     ctx(poptGetContext(NULL, argc, (char const **) uv.argv, options, 0),
         &my_poptFreeContext);
 
@@ -296,6 +333,8 @@ cpp_main(int argc, char ** argv)
     {
       app_state app;
 
+      app.set_prog_name(prog_name);
+
       while ((opt = poptGetNextOpt(ctx())) > 0)
         {
           if (local_options.find(opt) != local_options.end())
@@ -309,6 +348,12 @@ cpp_main(int argc, char ** argv)
 
             case OPT_QUIET:
               global_sanity.set_quiet();
+              ui.set_tick_writer(new tick_write_nothing);
+              break;
+
+            case OPT_REALLYQUIET:
+              global_sanity.set_reallyquiet();
+              ui.set_tick_writer(new tick_write_nothing);
               break;
 
             case OPT_NOSTD:
@@ -331,6 +376,10 @@ cpp_main(int argc, char ** argv)
               global_sanity.filename = system_path(argstr);
               break;
 
+            case OPT_LOG:
+              ui.redirect_log_to(system_path(argstr));
+              break;
+
             case OPT_DB_NAME:
               app.set_database(system_path(argstr));
               break;
@@ -344,12 +393,12 @@ cpp_main(int argc, char ** argv)
               break;
 
             case OPT_TICKER:
-              if (string(argstr) == "dot")
+              if (string(argstr) == "none" || global_sanity.quiet)
+                ui.set_tick_writer(new tick_write_nothing);
+              else if (string(argstr) == "dot")
                 ui.set_tick_writer(new tick_write_dot);
               else if (string(argstr) == "count")
                 ui.set_tick_writer(new tick_write_count);
-              else if (string(argstr) == "none")
-                ui.set_tick_writer(new tick_write_nothing);
               else
                 requested_help = true;
               break;
@@ -403,6 +452,10 @@ cpp_main(int argc, char ** argv)
               app.set_last(arglong);
               break;
 
+            case OPT_NEXT:
+              app.set_next(arglong);
+              break;
+
             case OPT_DEPTH:
               app.set_depth(arglong);
               break;
@@ -446,25 +499,55 @@ cpp_main(int argc, char ** argv)
             case OPT_EXTERNAL_DIFF:
               app.set_diff_format(external_diff);
               break;
-              
+
             case OPT_EXTERNAL_DIFF_ARGS:
               app.set_diff_args(utf8(string(argstr)));
               break;
 
-            case OPT_LCA:
-              app.use_lca = true;
-              break;
-
             case OPT_EXECUTE:
               app.execute = true;
               break;
 
+            case OPT_STDIO:
+              app.bind_stdio = true;
+              break;
+
+            case OPT_NO_TRANSPORT_AUTH:
+              app.use_transport_auth = false;
+              break;
+
             case OPT_BIND:
               {
-                std::string arg(argstr);
-                size_t colon = arg.find(':');
-                std::string addr_part = (colon == std::string::npos ? arg : arg.substr(0, colon));
-                std::string port_part = (colon == std::string::npos ? "" :  arg.substr(colon+1, arg.size() - colon));
+                string arg(argstr);
+                string addr_part, port_part;
+                size_t l_colon = arg.find(':');
+                size_t r_colon = arg.rfind(':');
+
+                // not an ipv6 address, as that would have at least two colons
+                if (l_colon == r_colon)
+                  {
+                    addr_part = (r_colon == string::npos ? arg : arg.substr(0, r_colon));
+                    port_part = (r_colon == string::npos ? "" :  arg.substr(r_colon+1, arg.size() - r_colon));
+                  }
+                else
+                  {
+                    // IPv6 addresses have a port specified in the style: [2001:388:0:13::]:80
+                    size_t squareb = arg.rfind(']');
+                    if ((arg.find('[') == 0) && (squareb != string::npos))
+                      {
+                        if (squareb < r_colon)
+                          port_part = (r_colon == string::npos ? "" :  arg.substr(r_colon+1, arg.size() - r_colon));
+                        else
+                          port_part = "";
+                        addr_part = (squareb == string::npos ? arg.substr(1, arg.size()) : arg.substr(1, squareb-1));
+                      }
+                    else
+                      {
+                        addr_part = arg;
+                        port_part = "";
+                      }
+                  }
+                app.bind_stdio = false;
                 app.bind_address = utf8(addr_part);
                 app.bind_port = utf8(port_part);
               }
@@ -485,6 +568,18 @@ cpp_main(int argc, char ** argv)
               }
               break;
 
+            case OPT_DROP_ATTR:
+              app.attrs_to_drop.insert(string(argstr));
+              break;
+
+            case OPT_NO_FILES:
+              app.no_files = true;
+              break;
+
+            case OPT_RECURSIVE:
+              app.set_recursive();
+              break;
+
             case OPT_HELP:
             default:
               requested_help = true;
@@ -513,13 +608,16 @@ cpp_main(int argc, char ** argv)
           throw usage(cmd);     // cmd may be empty, and that's fine.
         }
 
-      // at this point we allow a working copy (meaning search for it
-      // and if found read MT/options) but don't require it. certain
-      // commands may subsequently require a working copy or fail
+      // at this point we allow a workspace (meaning search for it
+      // and if found read _MTN/options, but don't use the data quite
+      // yet, and read all the monotonercs).  Processing the data
+      // from _MTN/options happens later.
+      // Certain commands may subsequently require a workspace or fail
+      // if we didn't find one at this point.
 
-      app.allow_working_copy();
+      app.allow_workspace();
 
-      // main options processed, now invoke the 
+      // main options processed, now invoke the
       // sub-command w/ remaining args
 
       if (cmd.empty())
@@ -556,24 +654,25 @@ cpp_main(int argc, char ** argv)
           if (command_options.find(o->val) != command_options.end())
             {
               o->argInfo &= ~POPT_ARGFLAG_DOC_HIDDEN;
-              L(F("Removed 'hidden' from option # %d\n") % o->argInfo);
+              L(FL("Removed 'hidden' from option # %d") % o->argInfo);
               count++;
             }
           else
             {
               o->argInfo |= POPT_ARGFLAG_DOC_HIDDEN;
-              L(F("Added 'hidden' to option # %d\n") % o->argInfo);
+              L(FL("Added 'hidden' to option # %d") % o->argInfo);
             }
         }
       free((void *)options[0].descrip); options[0].descrip = NULL;
       if (count != 0)
         {
           ostringstream sstr;
-          sstr << F("Options specific to 'monotone %s':") % u.which;
+          sstr << F("Options specific to '%s %s':")
+            % prog_name % u.which;
           options[0].descrip = strdup(sstr.str().c_str());
 
           options[0].argInfo |= POPT_ARGFLAG_DOC_HIDDEN;
-          L(F("Added 'hidden' to option # %d\n") % options[0].argInfo);
+          L(FL("Added 'hidden' to option # %d") % options[0].argInfo);
         }
 
       poptPrintHelp(ctx(), stdout, 0);
@@ -589,7 +688,7 @@ cpp_main(int argc, char ** argv)
     global_sanity.clean_shutdown = true;
     return 1;
   }
-  catch (std::ios_base::failure const & ex)
+  catch (ios_base::failure const & ex)
   {
     global_sanity.clean_shutdown = true;
     return 1;
@@ -598,3 +697,11 @@ cpp_main(int argc, char ** argv)
   global_sanity.clean_shutdown = true;
   return ret;
 }
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
