@@ -8,7 +8,7 @@ function temp_file()
    if tdir == nil then tdir = os.getenv("TMP") end
    if tdir == nil then tdir = os.getenv("TEMP") end
    if tdir == nil then tdir = "/tmp" end
-   return mkstemp(string.format("%s/mt.XXXXXX", tdir))
+   return mkstemp(string.format("%s/mtn.XXXXXX", tdir))
 end
 
 function execute(path, ...)   
@@ -24,8 +24,14 @@ end
 -- This is needed to work around some brokenness with some merge tools
 -- (e.g. on OS X)
 function execute_confirm(path, ...)   
-   execute(path, unpack(arg))
-   print(gettext("Press enter when the subprocess has completed"))
+   ret = execute(path, unpack(arg))
+
+   if (ret ~= 0)
+   then
+      print(gettext("Press enter"))
+   else
+      print(gettext("Press enter when the subprocess has completed"))
+   end
    io.read()
    return ret
 end
@@ -33,15 +39,14 @@ end
 -- attributes are persistent metadata about files (such as execute
 -- bit, ACLs, various special flags) which we want to have set and
 -- re-set any time the files are modified. the attributes themselves
--- are stored in a file .mt-attrs, in the working copy (and
--- manifest). each (f,k,v) triple in an attribute file turns into a
--- call to attr_functions[k](f,v) in lua.
+-- are stored in the roster associated with the revision. each (f,k,v)
+-- attribute triple turns into a call to attr_functions[k](f,v) in lua.
 
 if (attr_init_functions == nil) then
    attr_init_functions = {}
 end
 
-attr_init_functions["execute"] = 
+attr_init_functions["mtn:execute"] = 
    function(filename)
       if (is_executable(filename)) then 
         return "true" 
@@ -50,7 +55,7 @@ attr_init_functions["execute"] =
       end 
    end
 
-attr_init_functions["manual_merge"] = 
+attr_init_functions["mtn:manual_merge"] = 
    function(filename)
       if (binary_file(filename)) then 
         return "true" -- binary files must merged manually
@@ -63,23 +68,31 @@ if (attr_functions == nil) then
    attr_functions = {}
 end
 
-attr_functions["execute"] = 
+attr_functions["mtn:execute"] = 
    function(filename, value) 
       if (value == "true") then
          make_executable(filename)
       end
    end
 
+function dir_matches(name, dir)
+   -- helper for ignore_file, matching files within dir, or dir itself.
+   -- eg for dir of 'CVS', matches CVS/, CVS/*, */CVS/ and */CVS/*
+   if (string.find(name, "^" .. dir .. "/")) then return true end
+   if (string.find(name, "^" .. dir .. "$")) then return true end
+   if (string.find(name, "/" .. dir .. "/")) then return true end
+   if (string.find(name, "/" .. dir .. "$")) then return true end
+   return false
+end
 
 function ignore_file(name)
    -- project specific
    if (ignored_files == nil) then
       ignored_files = {}
-      local ignfile = io.open(".mt-ignore", "r")
+      local ignfile = io.open(".mtn-ignore", "r")
       if (ignfile ~= nil) then
          local line = ignfile:read()
-         while (line ~= nil)
-         do
+         while (line ~= nil) do
             table.insert(ignored_files, line)
             line = ignfile:read()
          end
@@ -88,89 +101,82 @@ function ignore_file(name)
    end
    for i, line in pairs(ignored_files)
    do
-      local pcallstatus, result = pcall(function() if (regex.search(line, name)) then return true else return false end end)
+      local pcallstatus, result = pcall(function() return regex.search(line, name) end)
       if pcallstatus == true then
           -- no error from the regex.search call
           if result == true then return true end
       else
-          -- regex.search had a problem, warn the user their .mt-ignore file syntax is wrong
-          io.stderr:write("WARNING: the line '" .. line .. "' in your .mt-ignore file caused exception '" .. result .. "'"
-                           .. " while matching filename '" .. name .. "', ignoring this regex for all remaining files.\n")
+          -- regex.search had a problem, warn the user their .mtn-ignore file syntax is wrong
+          io.stderr:write("WARNING: the line '" .. line .. "' in your .mtn-ignore file caused error '" .. result .. "'"
+                           .. " while matching filename '" .. name .. "'.\nignoring this regex for all remaining files.\n")
           table.remove(ignored_files, i)
       end
    end
-   -- c/c++
-   if (string.find(name, "%.a$")) then return true end
-   if (string.find(name, "%.so$")) then return true end
-   if (string.find(name, "%.o$")) then return true end
-   if (string.find(name, "%.la$")) then return true end
-   if (string.find(name, "%.lo$")) then return true end
-   if (string.find(name, "^core$")) then return true end
-   if (string.find(name, "/core$")) then return true end
-   if (string.find(name, "/core%.%d+$")) then return true end
-   -- python
-   if (string.find(name, "%.pyc$")) then return true end
-   if (string.find(name, "%.pyo$")) then return true end
-   -- TeX
-   if (string.find(name, "%.aux$")) then return true end
-   -- backup files
-   if (string.find(name, "%.bak$")) then return true end
-   if (string.find(name, "%.orig$")) then return true end
-   if (string.find(name, "%.rej$")) then return true end
-   if (string.find(name, "%~$")) then return true end
-   -- editor temp files
-   -- vim creates .foo.swp files
-   if (string.find(name, "%.[^/]*%.swp$")) then return true end
-   -- emacs creates #foo# files
-   if (string.find(name, "%#[^/]*%#$")) then return true end
-   -- autotools detritus:
-   if (string.find(name, "^autom4te.cache/")) then return true end
-   if (string.find(name, "/autom4te.cache/")) then return true end
-   if (string.find(name, "^.deps/")) then return true end
-   if (string.find(name, "/.deps/")) then return true end
-   -- Cons/SCons detritus:
-   if (string.find(name, "^.consign$")) then return true end
-   if (string.find(name, "/.consign$")) then return true end
-   if (string.find(name, "^.sconsign$")) then return true end
-   if (string.find(name, "/.sconsign$")) then return true end
-   -- other VCSes:
-   if (string.find(name, "^CVS/")) then return true end
-   if (string.find(name, "/CVS/")) then return true end
-   if (string.find(name, "^%.svn/")) then return true end
-   if (string.find(name, "/%.svn/")) then return true end
-   if (string.find(name, "^SCCS/")) then return true end
-   if (string.find(name, "/SCCS/")) then return true end
-   if (string.find(name, "^_darcs/")) then return true end
-   if (string.find(name, "^.cdv/")) then return true end
-   if (string.find(name, "^.git/")) then return true end
-   if (string.find(name, "%.scc$")) then return true end
-   -- desktop/directory configuration metadata
-   if (string.find(name, "^.DS_Store$")) then return true end
-   if (string.find(name, "/.DS_Store$")) then return true end
-   if (string.find(name, "^desktop.ini$")) then return true end
-   if (string.find(name, "/desktop.ini$")) then return true end
+
+   local file_pats = {
+      -- c/c++
+      "%.a$", "%.so$", "%.o$", "%.la$", "%.lo$", "^core$",
+      "/core$", "/core%.%d+$",
+      -- java
+      "%.class$",
+      -- python
+      "%.pyc$", "%.pyo$",
+      -- TeX
+      "%.aux$",
+      -- backup files
+      "%.bak$", "%.orig$", "%.rej$", "%~$",
+      -- vim creates .foo.swp files
+      "%.[^/]*%.swp$",
+      -- emacs creates #foo# files
+      "%#[^/]*%#$",
+      -- other VCSes (where metadata is stored in named files):
+      "%.scc$",
+      -- desktop/directory configuration metadata
+      "^.DS_Store$", "/.DS_Store$", "^desktop.ini$", "/desktop.ini$"
+   }
+
+   local dir_pats = {
+      -- autotools detritus:
+      "autom4te.cache", ".deps",
+      -- Cons/SCons detritus:
+      ".consign", ".sconsign",
+      -- other VCSes (where metadata is stored in named dirs):
+      "CVS", ".svn", "SCCS", "_darcs", ".cdv", ".git", ".bzr", ".hg"
+   }
+
+   for _, pat in ipairs(file_pats) do
+      if string.find(name, pat) then return true end
+   end
+   for _, pat in ipairs(dir_pats) do
+      if dir_matches(name, pat) then return true end
+   end
+
    return false;
 end
 
 -- return true means "binary", false means "text",
 -- nil means "unknown, try to guess"
 function binary_file(name)
-   local lowname=string.lower(name)
    -- some known binaries, return true
-   if (string.find(lowname, "%.gif$")) then return true end
-   if (string.find(lowname, "%.jpe?g$")) then return true end
-   if (string.find(lowname, "%.png$")) then return true end
-   if (string.find(lowname, "%.bz2$")) then return true end
-   if (string.find(lowname, "%.gz$")) then return true end
-   if (string.find(lowname, "%.zip$")) then return true end
+   local bin_pats = {
+      "%.gif$", "%.jpe?g$", "%.png$", "%.bz2$", "%.gz$", "%.zip$",
+      "%.class$", "%.jar$", "%.war$", "%.ear$"
+   }
+
    -- some known text, return false
-   if (string.find(lowname, "%.cc?$")) then return false end
-   if (string.find(lowname, "%.cxx$")) then return false end
-   if (string.find(lowname, "%.hh?$")) then return false end
-   if (string.find(lowname, "%.hxx$")) then return false end
-   if (string.find(lowname, "%.lua$")) then return false end
-   if (string.find(lowname, "%.texi$")) then return false end
-   if (string.find(lowname, "%.sql$")) then return false end
+   local txt_pats = {
+      "%.cc?$", "%.cxx$", "%.hh?$", "%.hxx$", "%.cpp$", "%.hpp$",
+      "%.lua$", "%.texi$", "%.sql$", "%.java$"
+   }
+
+   local lowname=string.lower(name)
+   for _, pat in ipairs(bin_pats) do
+      if string.find(lowname, pat) then return true end
+   end
+   for _, pat in ipairs(txt_pats) do
+      if string.find(lowname, pat) then return false end
+   end
+
    -- unknown - read file and use the guess-binary 
    -- monotone built-in function
    return guess_binary_file_contents(name)
@@ -193,7 +199,7 @@ function edit_comment(basetext, user_log_message)
 
    local tmp, tname = temp_file()
    if (tmp == nil) then return nil end
-   basetext = "MT: " .. string.gsub(basetext, "\n", "\nMT: ") .. "\n"
+   basetext = "MTN: " .. string.gsub(basetext, "\n", "\nMTN: ") .. "\n"
    if user_log_message == "" then
       tmp:write("\n")
    else
@@ -214,7 +220,7 @@ function edit_comment(basetext, user_log_message)
    local res = ""
    local line = tmp:read()
    while(line ~= nil) do 
-      if (not string.find(line, "^MT:")) then
+      if (not string.find(line, "^MTN:")) then
          res = res .. line .. "\n"
       end
       line = tmp:read()
@@ -225,11 +231,6 @@ function edit_comment(basetext, user_log_message)
 end
 
 
-function non_blocking_rng_ok()
-   return true
-end
-
-
 function persist_phrase_ok()
    return true
 end
@@ -263,7 +264,7 @@ function get_file_cert_trust(signers, id, name, val)
 end
 
 function accept_testresult_change(old_results, new_results)
-   local reqfile = io.open("MT/wanted-testresults", "r")
+   local reqfile = io.open("_MTN/wanted-testresults", "r")
    if (reqfile == nil) then return true end
    local line = reqfile:read()
    local required = {}
@@ -285,54 +286,43 @@ end
 
 -- merger support
 
-function merge2_meld_cmd(lfile, rfile)
-   return 
-   function()
-      return execute("meld", lfile, rfile)
-   end
-end
-
 function merge3_meld_cmd(lfile, afile, rfile)
    return 
    function()
-      return execute("meld", lfile, afile, rfile)
-   end
-end
-
-function merge2_tortoise_cmd(lfile, rfile, outfile)
-   return
-   function()
-      return execute("tortoisemerge",
-                     string.format("/theirs:%s", lfile),
-                     string.format("/mine:%s", rfile),
-                     string.format("/merged:%s", outfile))
+      local path = "meld"
+      local ret = execute(path, lfile, afile, rfile)
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+      end
+      return ret
    end
 end
 
 function merge3_tortoise_cmd(lfile, afile, rfile, outfile)
    return
    function()
-      return execute("tortoisemerge",
-                     string.format("/base:%s", afile),
-                     string.format("/theirs:%s", lfile),
-                     string.format("/mine:%s", rfile),
-                     string.format("/merged:%s", outfile))
-   end
-end
-
-function merge2_vim_cmd(vim, lfile, rfile, outfile)
-   return
-   function()
-      return execute(vim, "-f", "-d", "-c", string.format("file %s", outfile),
-                     lfile, rfile)
+      local path = "tortoisemerge"
+      local ret = execute(path,
+			  string.format("/base:%s", afile),
+			  string.format("/theirs:%s", lfile),
+			  string.format("/mine:%s", rfile),
+			  string.format("/merged:%s", outfile))
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+      end
+      return ret
    end
 end
 
 function merge3_vim_cmd(vim, afile, lfile, rfile, outfile)
    return
    function()
-      return execute(vim, "-f", "-d", "-c", string.format("file %s", outfile),
-                     afile, lfile, rfile)
+      local ret = execute(vim, "-f", "-d", "-c", string.format("file %s", outfile),
+			  afile, lfile, rfile)
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), vim))
+      end
+      return ret
    end
 end
 
@@ -347,17 +337,12 @@ function merge3_rcsmerge_vim_cmd(merge, vim, lfile, afile, rfile, outfile)
          copy_text_file(lfile, outfile);
          return 0
       end
-      return execute(vim, "-f", "-c", string.format("file %s", outfile),
-                     lfile)
-   end
-end
-
-function merge2_emacs_cmd(emacs, lfile, rfile, outfile)
-   local elisp = "(ediff-merge-files \"%s\" \"%s\" nil \"%s\")"
-   return 
-   function()
-      return execute(emacs, "--eval", 
-                     string.format(elisp, lfile, rfile, outfile))
+      local ret = execute(vim, "-f", "-c", string.format("file %s", outfile),
+			  lfile)
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), vim))
+      end
+      return ret
    end
 end
 
@@ -365,19 +350,12 @@ function merge3_emacs_cmd(emacs, lfile, afile, rfile, outfile)
    local elisp = "(ediff-merge-files-with-ancestor \"%s\" \"%s\" \"%s\" nil \"%s\")"
    return 
    function()
-      return execute(emacs, "--eval", 
-                     string.format(elisp, lfile, rfile, afile, outfile))
-   end
-end
-
-function merge2_xxdiff_cmd(left_path, right_path, merged_path, lfile, rfile, outfile)
-   return 
-   function()
-      return execute("xxdiff", 
-                     "--title1", left_path,
-                     "--title2", right_path,
-                     lfile, rfile, 
-                     "--merged-filename", outfile)
+      local ret = execute(emacs, "--eval", 
+			  string.format(elisp, lfile, rfile, afile, outfile))
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), emacs))
+      end
+      return ret
    end
 end
 
@@ -385,54 +363,54 @@ function merge3_xxdiff_cmd(left_path, anc_path, right_path, merged_path,
                            lfile, afile, rfile, outfile)
    return 
    function()
-      return execute("xxdiff", 
-                     "--title1", left_path,
-                     "--title2", right_path,
-                     "--title3", merged_path,
-                     lfile, afile, rfile, 
-                     "--merge", 
-                     "--merged-filename", outfile)
+      local path = "xxdiff"
+      local ret = execute(path, 
+			"--title1", left_path,
+			"--title2", right_path,
+			"--title3", merged_path,
+			lfile, afile, rfile, 
+			"--merge", 
+			"--merged-filename", outfile,
+			"--exit-with-merge-status")
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+      end
+      return ret
    end
 end
    
-function merge2_kdiff3_cmd(left_path, right_path, merged_path, lfile, rfile, outfile)
-   return 
-   function()
-      return execute("kdiff3", 
-                     "--L1", left_path,
-                     "--L2", right_path,
-                     lfile, rfile, 
-                     "-o", outfile)
-   end
-end
-
 function merge3_kdiff3_cmd(left_path, anc_path, right_path, merged_path, 
                            lfile, afile, rfile, outfile)
    return 
    function()
-      return execute("kdiff3", 
-                     "--L1", anc_path,
-                     "--L2", left_path,
-                     "--L3", right_path,
-                     afile, lfile, rfile, 
-                     "--merge", 
-                     "--o", outfile)
+      local path = "kdiff3"
+      local ret = execute(path, 
+			  "--L1", anc_path,
+			  "--L2", left_path,
+			  "--L3", right_path,
+			  afile, lfile, rfile, 
+			  "--merge", 
+			  "--o", outfile)
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+      end
+      return ret
    end
 end
 
-function merge2_opendiff_cmd(left_path, right_path, merged_path, lfile, rfile, outfile)
-   return 
-   function()
-      -- As opendiff immediately returns, let user confirm manually
-      return execute_confirm("opendiff",lfile,rfile,"-merge",outfile) 
-  end
-end
-
 function merge3_opendiff_cmd(left_path, anc_path, right_path, merged_path, lfile, afile, rfile, outfile)
    return 
    function()
+      local path = "opendiff"
       -- As opendiff immediately returns, let user confirm manually
-      execute_confirm("opendiff",lfile,rfile,"-ancestor",afile,"-merge",outfile)
+      local ret = execute_confirm(path,
+				  lfile,rfile,
+				  "-ancestor",afile,
+				  "-merge",outfile)
+      if (ret ~= 0) then
+	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+      end
+      return ret
    end
 end
 
@@ -476,102 +454,6 @@ function program_exists_in_path(program)
    return existsonpath(program) == 0
 end
 
-function get_preferred_merge2_command (tbl)
-   local cmd = nil
-   local left_path = tbl.left_path
-   local right_path = tbl.right_path
-   local merged_path = tbl.merged_path
-   local lfile = tbl.lfile
-   local rfile = tbl.rfile
-   local outfile = tbl.outfile 
-
-   local editor = os.getenv("EDITOR")
-   if editor ~= nil then editor = string.lower(editor) else editor = "" end
-
-   
-   if program_exists_in_path("kdiff3") then
-      cmd =   merge2_kdiff3_cmd (left_path, right_path, merged_path, lfile, rfile, outfile) 
-   elseif program_exists_in_path ("xxdiff") then 
-      cmd = merge2_xxdiff_cmd (left_path, right_path, merged_path, lfile, rfile, outfile) 
-   elseif program_exists_in_path ("opendiff") then 
-      cmd = merge2_opendiff_cmd (left_path, right_path, merged_path, lfile, rfile, outfile) 
-   elseif program_exists_in_path ("TortoiseMerge") then
-      cmd = merge2_tortoise_cmd(lfile, rfile, outfile)
-   elseif string.find(editor, "emacs") ~= nil or string.find(editor, "gnu") ~= nil then 
-      if string.find(editor, "xemacs") and program_exists_in_path("xemacs") then
-         cmd = merge2_emacs_cmd ("xemacs", lfile, rfile, outfile) 
-      elseif program_exists_in_path("emacs") then
-         cmd = merge2_emacs_cmd ("emacs", lfile, rfile, outfile) 
-      end
-   elseif string.find(editor, "vim") ~= nil then
-      io.write (string.format("\nWARNING: 'vim' was choosen to perform external 2-way merge.\n"..
-          "You should merge all changes to *LEFT* file due to limitation of program\n"..
-          "arguments.\n\n")) 
-      if os.getenv ("DISPLAY") ~= nil and program_exists_in_path ("gvim") then
-         cmd = merge2_vim_cmd ("gvim", lfile, rfile, outfile) 
-      elseif program_exists_in_path ("vim") then 
-         cmd = merge2_vim_cmd ("vim", lfile, rfile, outfile) 
-      end
-   elseif program_exists_in_path ("meld") then 
-      tbl.meld_exists = true 
-      io.write (string.format("\nWARNING: 'meld' was choosen to perform external 2-way merge.\n"..
-          "You should merge all changes to *LEFT* file due to limitation of program\n"..
-          "arguments.\n\n")) 
-      cmd = merge2_meld_cmd (lfile, rfile) 
-   end 
-   return cmd 
-end 
-
-function merge2 (left_path, right_path, merged_path, left, right) 
-   local ret = nil 
-   local tbl = {}
-
-   tbl.lfile = nil 
-   tbl.rfile = nil 
-   tbl.outfile = nil 
-   tbl.meld_exists = false
- 
-   tbl.lfile = write_to_temporary_file (left) 
-   tbl.rfile = write_to_temporary_file (right) 
-   tbl.outfile = write_to_temporary_file ("") 
-
-   if tbl.lfile ~= nil and tbl.rfile ~= nil and tbl.outfile ~= nil 
-   then 
-      tbl.left_path = left_path 
-      tbl.right_path = right_path 
-      tbl.merged_path = merged_path 
-
-      local cmd = get_preferred_merge2_command (tbl) 
-
-      if cmd ~=nil 
-      then 
-         io.write (string.format(gettext("executing external 2-way merge command\n")))
-         cmd ()
-         if tbl.meld_exists 
-         then 
-            ret = read_contents_of_file (tbl.lfile, "r")
-         else
-            ret = read_contents_of_file (tbl.outfile, "r") 
-         end 
-         if string.len (ret) == 0 
-         then 
-            ret = nil 
-         end
-      else
-         io.write (string.format("No external 2-way merge command found.\n"..
-            "You may want to check that $EDITOR is set to an editor that supports 2-way merge,\n"..
-            "set this explicitly in your get_preferred_merge2_command hook,\n"..
-            "or add a 2-way merge program to your path.\n\n"))
-      end
-   end
-
-   os.remove (tbl.lfile)
-   os.remove (tbl.rfile)
-   os.remove (tbl.outfile)
-   
-   return ret
-end
-
 function get_preferred_merge3_command (tbl)
    local cmd = nil
    local left_path = tbl.left_path
@@ -654,17 +536,22 @@ function merge3 (anc_path, left_path, right_path, merged_path, ancestor, left, r
       if cmd ~=nil 
       then 
          io.write (string.format(gettext("executing external 3-way merge command\n")))
-         cmd ()
-         if tbl.meld_exists 
-         then 
-            ret = read_contents_of_file (tbl.afile, "r")
-         else
-            ret = read_contents_of_file (tbl.outfile, "r") 
-         end 
-         if string.len (ret) == 0 
-         then 
-            ret = nil 
-         end
+	 -- cmd() return 0 on success.
+         if cmd () ~= 0
+	 then
+	    ret = nil
+	 else
+	    if tbl.meld_exists 
+	    then 
+	       ret = read_contents_of_file (tbl.afile, "r")
+	    else
+	       ret = read_contents_of_file (tbl.outfile, "r") 
+	    end 
+	    if string.len (ret) == 0 
+	    then 
+	       ret = nil 
+	    end
+	 end
       else
          io.write (string.format("No external 3-way merge command found.\n"..
             "You may want to check that $EDITOR is set to an editor that supports 3-way merge,\n"..
@@ -735,7 +622,7 @@ function expand_date(str)
       return os.date("%FT%T", t)
    end
    
-         -- today don't uses the time
+   -- today don't uses the time         # for xgettext's sake, an extra quote
    if str == "today"
    then
       local t = os.time(os.date('!*t'))
@@ -805,6 +692,7 @@ function get_netsync_read_permitted(branch, ident)
    local permfile = io.open(get_confdir() .. "/read-permissions", "r")
    if (permfile == nil) then return false end
    local dat = permfile:read("*a")
+   io.close(permfile)
    local res = parse_basic_io(dat)
    if res == nil then
       io.stderr:write("file read-permissions cannot be parsed\n")
@@ -818,16 +706,20 @@ function get_netsync_read_permitted(branch, ident)
       if item.name == "pattern" then
          if matches and not cont then return false end
          matches = false
+	 cont = false
          for j, val in pairs(item.values) do
             if globish_match(val, branch) then matches = true end
          end
       elseif item.name == "allow" then if matches then
          for j, val in pairs(item.values) do
             if val == "*" then return true end
+	    if val == "" and ident == nil then return true end
             if globish_match(val, ident) then return true end
          end
       end elseif item.name == "deny" then if matches then
          for j, val in pairs(item.values) do
+            if val == "*" then return false end
+	    if val == "" and ident == nil then return false end
             if globish_match(val, ident) then return false end
          end
       end elseif item.name == "continue" then if matches then
@@ -848,13 +740,92 @@ function get_netsync_write_permitted(ident)
    if (permfile == nil) then
       return false
    end
+   local matches = false
    local line = permfile:read()
-   while (line ~= nil) do
+   while (not matches and line ~= nil) do
       local _, _, ln = string.find(line, "%s*([^%s]*)%s*")
-      if ln == "*" then return true end
-      if globish_match(ln, ident) then return true end
+      if ln == "*" then matches = true end
+      if globish_match(ln, ident) then matches = true end
       line = permfile:read()
    end
    io.close(permfile)
-   return false
+   return matches
+end
+
+-- This is a simple funciton which assumes you're going to be spawning
+-- a copy of mtn, so reuses a common bit at the end for converting
+-- local args into remote args. You might need to massage the logic a
+-- bit if this doesn't fit your assumptions.
+
+function get_netsync_connect_command(uri, args)
+
+	local argv = nil
+	local quote_patterns = false
+
+	if uri["scheme"] == "ssh" 
+		and uri["host"] 
+		and uri["path"] then
+
+		argv = { "ssh" }
+		if uri["user"] then
+			table.insert(argv, "-l")
+			table.insert(argv, uri["user"])
+		end
+		if uri["port"] then
+			table.insert(argv, "-p")
+			table.insert(argv, uri["port"])
+		end
+
+		table.insert(argv, uri["host"])
+    quote_patterns = true
+	end
+	
+	if uri["scheme"] == "file" and uri["path"] then
+		argv = { }
+	end
+
+	if argv then
+
+		table.insert(argv, "mtn")
+
+		if args["debug"] then
+			table.insert(argv, "--debug")
+		else
+			table.insert(argv, "--quiet")
+		end
+
+		table.insert(argv, "--db")
+		table.insert(argv, uri["path"])
+		table.insert(argv, "serve")
+		table.insert(argv, "--stdio")
+		table.insert(argv, "--no-transport-auth")
+
+		-- patterns must be quoted to avoid a remote shell expanding them
+		if args["include"] then
+			local include = args["include"]
+			if quote_patterns then
+				include = "'" .. args["include"] .. "'"
+			end
+			table.insert(argv, include)
+		end
+
+		if args["exclude"] then
+			table.insert(argv, "--exclude")
+			local exclude = args["exclude"]
+			if quote_patterns then
+				exclude = "'" .. args["exclude"] .. "'"
+			end
+			table.insert(argv, exclude)
+		end
+	end
+	return argv
+end
+
+function use_transport_auth(uri)
+	if uri["scheme"] == "ssh" 
+	or uri["scheme"] == "file" then
+		return false
+	else
+		return true
+	end
 end
